"""Database connection management"""

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, scoped_session, Session
from .base import Base

DATABASE_FILENAME = "project.db"

engine = None
session_factory = None
scoped_session_registry = None


def init_database(db_path: str = DATABASE_FILENAME, create_tables: bool = True) -> None:
    global engine, session_factory, scoped_session_registry

    if engine is None:
        engine = create_engine(
            f"sqlite:///{db_path}",
            connect_args={"check_same_thread": False},
        )
        print(f"Database engine created for {db_path}", flush=True)
        session_factory = sessionmaker(bind=engine)
        scoped_session_registry = scoped_session(session_factory)
        if create_tables:
            Base.metadata.create_all(engine)
            print(f"Database tables created for {db_path}", flush=True)
        else:
            print(f"Database connected (tables not created) for {db_path}", flush=True)
    else:
        print(f"Database engine already exists for {db_path}, reusing", flush=True)


def get_session() -> Session:
    if scoped_session_registry is None:
        raise RuntimeError()
    return scoped_session_registry()



from typing import Literal, get_args, cast

GeodePointMeshType = Literal[
    "PointSet2D",
    "PointSet3D",
]
GeodeEdgeMeshType = Literal[
    "EdgedCurve2D",
    "EdgedCurve3D",
]
GeodePolygonMeshType = Literal[
    "RasterImage2D",
    "PolygonalSurface2D",
    "PolygonalSurface3D",
    "TriangulatedSurface2D",
    "TriangulatedSurface3D",
    "RegularGrid2D",
    "LightRegularGrid2D",
]
GeodePolyhedronMeshType = Literal[
    "RasterImage3D",
    "PolyhedralSolid3D",
    "TetrahedralSolid3D",
    "HybridSolid3D",
    "RegularGrid3D",
    "LightRegularGrid3D",
]
GeodeMeshType = (
    Literal[
        "VertexSet",
        "Graph",
    ]
    | GeodePointMeshType
    | GeodeEdgeMeshType
    | GeodePolygonMeshType
    | GeodePolyhedronMeshType
)
GeodeModelType = Literal[
    "BRep",
    "Section",
    "StructuralModel",
    "CrossSection",
    "ImplicitStructuralModel",
    "ImplicitCrossSection",
]
GeodeObjectType = GeodeMeshType | GeodeModelType


def _flatten_literal_args(literal: object) -> tuple[str, ...]:
    flattened: list[str] = []
    for arg in get_args(literal):
        if isinstance(arg, str):
            flattened.append(arg)
        else:
            flattened.extend(_flatten_literal_args(arg))
    return tuple(flattened)


GeodeObjectType_values = _flatten_literal_args(GeodeObjectType)


def geode_object_type(value: str) -> GeodeObjectType:
    if value not in GeodeObjectType_values:
        raise ValueError(
            f"Invalid GeodeObjectType: {value!r}. Must be one of {GeodeObjectType_values}"
        )
    return cast(GeodeObjectType, value)


ViewerType = Literal["mesh", "model"]

ViewerElementsType = Literal["points", "edges", "polygons", "polyhedra", "default"]



import os
import glob
import json

type SchemaDict = dict[str, str]


def get_schemas_dict(path: str) -> dict[str, SchemaDict]:
    schemas_dict: dict[str, SchemaDict] = {}
    for json_file in glob.glob(os.path.join(path, "*.json")):
        filename = os.path.basename(json_file)
        with open(os.path.join(path, json_file), "r") as file:
            file_content = json.load(file)
            schemas_dict[os.path.splitext(filename)[0]] = file_content
    return schemas_dict



from opengeodeweb_microservice.database.data import Data
from opengeodeweb_microservice.database.connection import get_session


def test_data_crud_operations(clean_database: None) -> None:
    data = Data.create(
        geode_object="test_object",
        viewer_object="test_viewer",
        viewer_elements_type="test_type",
    )
    print("id", data.id, flush=True)
    assert data.id is not None
    assert isinstance(data.id, str)

    retrieved = Data.get(data.id)
    assert retrieved is not None
    assert isinstance(retrieved, Data)
    assert retrieved.geode_object == "test_object"
    assert retrieved.id == data.id
    non_existent = Data.get("fake_id")
    assert non_existent is None


def test_data_with_file_assignments(clean_database: None) -> None:
    data = Data.create(
        geode_object="geode_object",
        viewer_object="viewer_object",
        viewer_elements_type="test_type",
    )
    data_id = data.id
    data.native_file = "native.og_brep"
    data.viewable_file = "viewable.vtm"
    data.light_viewable_file = "light.vtp"

    with get_session() as session:
        session.add(data)
        session.commit()

    retrieved = Data.get(data_id)
    assert retrieved is not None
    assert retrieved.native_file == "native.og_brep"
    assert retrieved.viewable_file == "viewable.vtm"
    assert retrieved.light_viewable_file == "light.vtp"
    assert retrieved.geode_object == "geode_object"



from sqlalchemy import String, JSON, select
from sqlalchemy.orm import Mapped, mapped_column
from .connection import get_session
from .base import Base
from .data_types import GeodeObjectType, ViewerType, ViewerElementsType
import uuid


class Data(Base):
    __tablename__ = "datas"

    id: Mapped[str] = mapped_column(
        String, primary_key=True, default=lambda: str(uuid.uuid4()).replace("-", "")
    )
    geode_object: Mapped[GeodeObjectType] = mapped_column(String, nullable=False)
    viewer_object: Mapped[ViewerType] = mapped_column(String, nullable=False)
    viewer_elements_type: Mapped[ViewerElementsType] = mapped_column(
        String, nullable=False
    )
    native_file: Mapped[str | None] = mapped_column(String, nullable=True)
    viewable_file: Mapped[str | None] = mapped_column(String, nullable=True)
    light_viewable_file: Mapped[str | None] = mapped_column(String, nullable=True)

    @staticmethod
    def create(
        geode_object: GeodeObjectType,
        viewer_object: ViewerType,
        viewer_elements_type: ViewerElementsType,
    ) -> "Data":
        data_entry = Data(
            geode_object=geode_object,
            viewer_object=viewer_object,
            viewer_elements_type=viewer_elements_type,
        )

        session = get_session()
        session.add(data_entry)
        session.flush()
        return data_entry

    @staticmethod
    def get(data_id: str) -> "Data | None":
        session = get_session()
        data_query = select(Data).where(Data.id == data_id)
        return session.scalars(data_query).first()
